package cc.smtweb.system.bpm.web.design.form;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.jdbc.AbsDbWorker;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.mvc.service.DefaultSaveHandler;
import cc.smtweb.framework.core.util.CommUtil;
import cc.smtweb.framework.core.util.JsonUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/5/9 17:05
 * 页面定义保存，注意不要覆盖content和dataset
 */
public class ModelFormSaveHandler extends DefaultSaveHandler<ModelForm> {
    //自动生成的模块子页面，暂存，便于维护缓存
    private List<ModelForm> listFormChild;

    public ModelFormSaveHandler() {
        super(ModelForm.ENTITY_NAME);
    }

    @Override
    protected void updateBean(EntityDao<ModelForm> dao) {
        updateFormChild(dao);
        dao.updateEntityEx(bean, "mf_content", "mf_dataset", "mf_tmpl");
    }

    @Override
    protected void insertBean(EntityDao<ModelForm> dao) {
        super.insertBean(dao);
        updateFormChild(dao);
    }

    private void updateFormChild(EntityDao<ModelForm> dao) {
        SwMap opts = bean.getOpts();
        if (opts != null) {
            opts.remove(ModelFormHelper.KEY_EVENT_PATH);
            bean.setOption(JsonUtil.encodeString(opts));
        }
        if (bean.getType() == SwEnum.FormType.MODULE.value) {
            addChildPage(dao, opts);
        }
    }

    //自动增加子页面
    private void addChildPage(EntityDao<ModelForm> dao, SwMap opts) {
        listFormChild = new ArrayList<>();
        Set<ModelForm> setExists = ModelFormCache.getInstance().getListByModule(bean.getEntityId());
        //维护已有的子页面
        if(setExists!=null){
            for (ModelForm mf: setExists) {
                if (!CommUtil.isStrEquals(mf.getService(), bean.getService())) {
                    mf.setService(bean.getService());
                    dao.updateEntityEx(bean, "mf_content", "mf_dataset", "mf_tmpl");
                    listFormChild.add(mf);
                }
            }
        }
        if (opts == null) return;
        List<String> incPage = (List<String>) opts.get("incPage");
        if (incPage == null || incPage.isEmpty()) return;

        //增加新的子页面
        for (String type : incPage) {
            String name = bean.getName() + "_" + type;
            if (existsPage(setExists, name)) continue;
            ModelForm page = new ModelForm();
            page.getData().putAll(bean.getData());
            page.setId(DbEngine.getInstance().nextId());
            page.setOption("{}");
            page.setDataset(null);
            page.setContent(null);
            page.setTmpl(null);
            page.setType(SwEnum.FormType.PAGE.value);
            page.setName(name);
            page.setParent(bean.getId());
            page.setTitle(bean.getTitle() + "_" + SwEnum.PageType.instance.getName(type));
            listFormChild.add(page);
            dao.insertEntity(page);
        }
    }

    @Override
    protected void saveSuccess() {
        ModelFormCache.getInstance().put(bean);
        if (listFormChild != null) {
            for (ModelForm page : listFormChild) {
                ModelFormCache.getInstance().put(page);
            }
        }
    }

    @Override
    protected void saveFailed() {
        ModelFormCache.getInstance().reset(bean);
        if (listFormChild != null) {
            for (ModelForm page : listFormChild) {
                ModelFormCache.getInstance().reset(page);
            }
        }
    }

    //已经存在的，不管了
    private boolean existsPage(Set<ModelForm> setExists, String name) {
        if (setExists == null){
           return false;
        }
        for (ModelForm form : setExists) {
            if (name.equalsIgnoreCase(form.getName())) return true;
        }
        return false;
    }

    public R saveDataset() {
        long id = params.readLong("id");
        String data = params.readString("data");
        bean = loadComp(id);
        if (StringUtils.isEmpty(data)) {
            throw new BizException("没有待保存的数据！");
        }
        bean.setDataset(ModelFormHelper.buildSaveDataset(data));
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(tableName);

                ModelTable table = ModelTableCache.getInstance().getByName(tableName);
                ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
                if (field != null) bean.put(field.getName(), us.getUserId());
                dao.updateEntity(bean, "mf_dataset");
            }

            @Override
            public void doAfterDbCommit() {
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success();
    }

    public R saveModel() {
        long id = params.readLong("id");
        String data = params.readString("data");
        bean = loadComp(id);
        if (StringUtils.isEmpty(data)) {
            throw new BizException("没有待保存的数据！");
        }
        bean.setContent(ModelFormHelper.buildSaveModel(bean, data));
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(tableName);

                ModelTable table = ModelTableCache.getInstance().getByName(tableName);
                ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
                if (field != null) bean.put(field.getName(), us.getUserId());
                dao.updateEntity(bean, "mf_content");
            }

            @Override
            public void doAfterDbCommit() {
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success();
    }

    public R saveModelByTmpl() {
        long id = params.readLong("pageId");
        String templateId = params.readString("templateId");
        String data = params.readString("data");
        bean = loadComp(id);
        if (StringUtils.isEmpty(data)) {
            throw new BizException("没有待保存的数据！");
        }
        bean.setTmpl(data);
        ModelFormHelper.buildSaveModelByTmpl(bean, templateId);
        DbEngine.getInstance().doTrans(new AbsDbWorker() {
            @Override
            public void work() {
                EntityDao<ModelForm> dao = DbEngine.getInstance().findDao(tableName);

                ModelTable table = ModelTableCache.getInstance().getByName(tableName);
                ModelField field = table.findFieldByType(SwEnum.FieldType.UPDATE_USER.value);
                if (field != null) bean.put(field.getName(), us.getUserId());
                dao.updateEntity(bean, "mf_content,mf_tmpl");
            }

            @Override
            public void doAfterDbCommit() {
                saveSuccess();
            }

            @Override
            public void doAfterDbRollback() {
                saveFailed();
            }
        });
        return R.success(ModelFormHelper.buildReqModel(bean));
    }
}
